<?php

namespace Drupal\translate_side_by_side\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Language\Language;
use Drupal\Core\Language\LanguageManager;
use Drupal\node\Entity\Node;
use Drupal\block_content\Entity\BlockContent;
use Drupal\system\Entity\Menu;
use Drupal\Core\Menu\MenuTreeParameters;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\taxonomy\Entity\Vocabulary;
use Drupal\taxonomy\Entity\Term;

/**
 * Defines a form to configure module settings.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'translate_side_by_side.settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['config.translate_side_by_side'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $config = \Drupal::config('config.translate_side_by_side');

    $form = [];

    $form['selection'] = [
      '#title' => $this->t('Source language: @name',
      ['@name' => \Drupal::languageManager()->getDefaultLanguage()->getName()]),
      '#type' => 'fieldset',
    ];
    $form['selection']['target'] = [
      '#title' => $this->t('Target Language'),
      '#type' => 'language_select',
      '#languages' => Language::STATE_CONFIGURABLE,
    ];
    $form['selection']['filluntranslated'] = [
      '#title' => $this->t('Fill untranslated with source'),
      '#type' => 'checkbox',
    ];
    $form['selection']['load'] = [
      '#type' => 'button',
      '#value' => $this->t('Load'),
    ];

    $target = ($form_state->getValue('target') != '') ? ($form_state
      ->getValue('target')) : (\Drupal::languageManager()->getDefaultLanguage()
      ->getId());
    $form['menus'] = [
      '#value' => $this->t('Menus'),
      '#type' => 'html_tag',
      '#tag' => 'h3',
    ];
    $form = $this->buildFormMenu($form, $form_state, $target);
    $form['nodes'] = [
      '#value' => $this->t('Nodes'),
      '#type' => 'html_tag',
      '#tag' => 'h3',
    ];
    $form = $this->buildFormNode($form, $form_state, $target);
    $form['blocks'] = [
      '#value' => $this->t('Blocks'),
      '#type' => 'html_tag',
      '#tag' => 'h3',
    ];
    $form = $this->buildFormBlock($form, $form_state, $target);
    $form['taxonomy'] = [
      '#value' => $this->t('Taxonomy'),
      '#type' => 'html_tag',
      '#tag' => 'h3',
    ];
    $form = $this->buildFormTaxonomy($form, $form_state, $target);

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  private function buildFormNode(
  array $form,
  FormStateInterface $form_state,
  string $target) {
    $nids = \Drupal::entityQuery('node')
      ->condition('langcode', 'zzz', '<>')
      ->condition('langcode', 'zxx', '<>')
      ->sort('title', 'ASC', 'en')
      ->execute();
    $nodes = Node::loadMultiple($nids);
    foreach ($nodes as $onenode) {
      if ($onenode->hasTranslation($target)) {
        $translation = $onenode->getTranslation($target);
      }
      else {
        $translation = NULL;
      }
      $html = '<br><strong>' . $onenode->title->value . ':</strong> ' .
      str_replace('/' . \Drupal::languageManager()->getDefaultLanguage()
        ->getId(), '', $onenode->toUrl()->toString()) .
      ' <span style="color:#CCCCCC">(node ID ' .
      $onenode->nid->value . ')</span><br>';
      $html .= '<table border="1" width="100%"><thead><tr><th width="20%">Field</th><th width="40%">' .
      $onenode->language()->getName() . '</th><th width="40%">' .
      \Drupal::languageManager()->getLanguage($target)->getName() .
      '</th></tr></thead>';

      $fieldlist = [];
      $entityManager = \Drupal::service('entity_field.manager');
      $fields = $entityManager->getFieldDefinitions('node', $onenode
        ->getType());

      foreach ($fields as $field_name => $field_definition) {
        $field_display = entity_get_display('node', $onenode->getType(),
        'default')->getComponent($field_name);

        if (($field_name == 'title' || $field_name == 'body' ||
        substr($field_name, 0, 6) === 'field_')) {
          foreach ($onenode->get($field_name)->getValue() as $onekey =>
          $onevalue) {
            $txtattr = NULL;

            if ($field_definition->getType() == 'image') {
              $txtattr = 'alt';
            }
            elseif ($field_definition->getType() == 'link') {
              $txtattr = 'title';
            }
            elseif (strpos($field_definition->getType(), 'string') !== FALSE
            || strpos($field_definition->getType(), 'text') !== FALSE) {
              $txtattr = 'value';
            }
            if ($translation !== NULL) {
              $field_val = $translation->get($field_name)
                ->getValue()[$onekey][$txtattr];
            }
            elseif ($form_state->getValue('filluntranslated') === 0) {
              $field_val = '';
            }
            else {
              $field_val = $onevalue[$txtattr];
            }

            if ($txtattr !== NULL) {
              $fieldlist[$field_display['weight'] * 1000 + $onekey] = [
                'f' => $field_name,
                's' => $onevalue[$txtattr],
                't' => $field_val,
              ];
            }
          }
        }
      }
      ksort($fieldlist);
      foreach ($fieldlist as $onefieldlist) {
        $html .= '<tr valign="top"><td style="color:#CCCCCC"><small>' .
        $onefieldlist['f'] . '</small></td><td lang="en">' . $onefieldlist['s'] .
        '</td><td lang="' . $target . '">' . $onefieldlist['t'] . '</td></tr>';
      }
      $html .= '</table><br>';
      $form['tablenodes'][]['table'] = [
        '#type' => 'item',
        '#children' => $html,
      ];
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  private function buildFormBlock(array $form, FormStateInterface $form_state, $target) {
    $bids = \Drupal::entityQuery('block_content')
      ->condition('langcode', 'zzz', '<>')
      ->condition('langcode', 'zxx', '<>')
      ->sort('info', 'ASC', 'en')
      ->execute();
    $blocks = BlockContent::loadMultiple($bids);

    foreach ($blocks as $oneblock) {
      if ($oneblock->hasTranslation($target)) {
        $translation = $oneblock->getTranslation($target);
      }
      else {
        $translation = NULL;
      }
      $html = '<br><strong>' . $oneblock->info->value .
      ':</strong> <span style="color:#CCCCCC">(block ID ' .
      $oneblock->id->value . ')</span><br>';
      $html .= '<table border="1" width="100%"><thead><tr><th width="20%">Field</th><th width="40%">' .
      $oneblock->language()->getName() . '</th><th width="40%">' .
      \Drupal::languageManager()->getLanguage($target)->getName() .
      '</th></tr></thead>';
      if ($translation !== NULL) {
        $info_val = $translation->get('info')->getValue()[0]['value'];
        $body_val = $translation->get('body')->getValue()[0]['value'];
      }
      elseif ($form_state->getValue('filluntranslated') === 0) {
        $info_val = '';
        $body_val = '';
      }
      else {
        $info_val = $oneblock->info->value;
        $body_val = $oneblock->body->value;
      }

      $fieldlist = [];
      $fieldlist[0] = [
        'f' => 'info',
        's' => $oneblock->info->value,
        't' => $info_val,
      ];
      $fieldlist[1] = [
        'f' => 'body',
        's' => $oneblock->body->value,
        't' => $body_val,
      ];

      foreach ($fieldlist as $onefieldlist) {
        $html .= '<tr valign="top"><td style="color:#CCCCCC"><small>' .
        $onefieldlist['f'] . '</small></td><td lang="en">' .
        $onefieldlist['s'] . '</td><td lang="' . $target . '">' .
        $onefieldlist['t'] . '</td></tr>';
      }

      $html .= '</table><br>';
      $form['tableblocks'][]['table'] = [
        '#type' => 'item',
        '#children' => $html,
      ];
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  private function buildFormMenu(array $form, FormStateInterface $form_state, $target) {
    $mids = \Drupal::entityQuery('menu')
      ->condition('langcode', 'zzz', '<>')
      ->condition('langcode', 'zxx', '<>')
      ->condition('id', 'account', '<>')
      ->condition('id', 'admin', '<>')
      ->condition('id', 'devel', '<>')
      ->condition('id', 'tools', '<>')
      ->sort('label', 'ASC', 'en')
      ->execute();
    $menus = Menu::loadMultiple($mids);

    foreach ($menus as $onemenu) {
      $html = '<br><strong>' . $onemenu->get('label') . ':</strong> <span style="color:#CCCCCC">(Machine name: ' .
      $onemenu->get('id') . ')</span><br>';
      $html .= '<table border="1" width="100%"><thead><tr><th width="20%">ID</th><th width="40%">' .
      $onemenu->language()->getName() . '</th><th width="40%">' .
      \Drupal::languageManager()->getLanguage($target)->getName() .
      '</th></tr></thead>';

      $fieldlist = [];

      $menu_tree = \Drupal::service('menu.link_tree');
      $parameters = new MenuTreeParameters();
      $parameters->setMinDepth(0)->setMaxDepth(4)->onlyEnabledLinks();
      $tree = $menu_tree->load($onemenu->get('id'), $parameters);
      $manipulators = [
        ['callable' => 'menu.default_tree_manipulators:generateIndexAndSort'],
        ['callable' => 'menu.default_tree_manipulators:flatten'],
      ];
      $tree = $menu_tree->transform($tree, $manipulators);
      $subtrees = [];
      $cacheability = new CacheableMetadata();
      foreach ($tree as $element) {
        /** @var \Drupal\Core\Menu\MenuLinkInterface $link */
        $link = $element->link;
        $storage = \Drupal::entityTypeManager()
          ->getStorage('menu_link_content');

        $entity_id = $element->link
          ->getPluginDefinition()['metadata']['entity_id'];
        $entity = $storage->load($entity_id);
        $langcode_key = $entity->getEntityType()->getKey('langcode');

        if ($entity->isTranslatable() && $entity->hasTranslation($target)) {
          $translation = $entity->getTranslation($target);
        }
        else {
          $translation = NULL;
        }
        if ($translation !== NULL) {
          $menu_val = $entity->getTranslation($target)->getTitle();
        }
        elseif ($form_state->getValue('filluntranslated') === 0) {
          $menu_val = '';
        }
        else {
          $menu_val = $entity->getTitle();
        }

        $fieldlist[] = [
          'f' => $entity_id,
          's' => $entity->getTitle(),
          't' => $menu_val,
        ];
      }

      foreach ($fieldlist as $onefieldlist) {
        $html .= '<tr valign="top"><td style="color:#CCCCCC"><small>' .
        $onefieldlist['f'] . '</small></td><td lang="en">' .
        $onefieldlist['s'] . '</td><td lang="' . $target . '">' .
        $onefieldlist['t'] . '</td></tr>';
      }

      $html .= '</table><br>';
      $form['tablemenus'][]['table'] = [
        '#type' => 'item',
        '#children' => $html,
      ];
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  private function buildFormTaxonomy(array $form, FormStateInterface $form_state, $target) {
    $vocabularies = Vocabulary::loadMultiple();

    foreach ($vocabularies as $onevocabulary) {
      $html = '<br><strong>' . $onevocabulary->get('name') .
      ':</strong> <span style="color:#CCCCCC">(Machine name: ' .
      $onevocabulary->get('vid') . ')</span><br>';

      $tids = \Drupal::entityQuery('taxonomy_term')->condition('vid',
      $onevocabulary->get('vid'))->execute();
      $terms = Term::loadMultiple($tids);

      foreach ($terms as $oneterm) {
        $fieldlist = [];
        $html .= '<table border="1" width="100%"><thead><tr><th width="20%">Field</th><th width="40%">' .
        $onevocabulary->language()->getName() . '</th><th width="40%">' .
        \Drupal::languageManager()->getLanguage($target)->getName() .
        '</th></tr></thead>';
        if ($oneterm->isTranslatable() && $oneterm->hasTranslation($target)) {
          $translation = $oneterm->getTranslation($target);
        }
        else {
          $translation = NULL;
        }
        if ($translation !== NULL) {
          $name_val = $oneterm->getTranslation($target)->getName();
          $desc_val = $oneterm->getTranslation($target)->getDescription();
        }
        elseif ($form_state->getValue('filluntranslated') === 0) {
          $name_val = '';
          $desc_val = '';
        }
        else {
          $name_val = $oneterm->getName();
          $desc_val = $oneterm->getDescription();
        }

        $fieldlist[] = [
          'f' => 'name',
          's' => $oneterm->getName(),
          't' => $name_val,
        ];
        $fieldlist[] = [
          'f' => 'description',
          's' => $oneterm->getDescription(),
          't' => $desc_val,
        ];

        foreach ($fieldlist as $onefieldlist) {
          $html .= '<tr valign="top"><td style="color:#CCCCCC"><small>' .
          $onefieldlist['f'] . '</small></td><td lang="en">' .
          $onefieldlist['s'] . '</td><td lang="' . $target . '">' .
          $onefieldlist['t'] . '</td></tr>';
        }
        $html .= '</table><br>';
      }

      $html .= '</table><br>';
      $form['tabletaxonomy'][]['table'] = [
        '#type' => 'item',
        '#children' => $html,
      ];
    }
    return $form;
  }
  
}
